#include <BulkFileIO.h>

#include <iostream>
#include <windows.h>

namespace BulkFileIO
{
	void* FileMemAllocator::PlatAllocate(long numBytes)
	{
		LPVOID result = VirtualAlloc(NULL, numBytes, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
		ASSERT(result != NULL);

		return result;
	}

	void FileMemAllocator::PlatFree(void* mem)
	{
		BOOL result = VirtualFree(mem, 0, MEM_RELEASE);
		ASSERT(result == TRUE);
	}



	LoadResults LoadFileToMemory(const ResourceIdentifier& resId, FileMemAllocator& alloc)
	{
		LoadResults ret;
		ret.data = 0;
		ret.dataSize = -1;

		HANDLE fileHandle = CreateFile(resId.GetFilepath().string().c_str(), 
										GENERIC_READ, 
										FILE_SHARE_READ, 
										NULL, 
										OPEN_EXISTING, 
		//								FILE_FLAG_NO_BUFFERING | FILE_FLAG_OVERLAPPED | FILE_FLAG_SEQUENTIAL_SCAN,
										FILE_FLAG_NO_BUFFERING | FILE_FLAG_SEQUENTIAL_SCAN,
										NULL);
		if(fileHandle == INVALID_HANDLE_VALUE) return ret;

		// Figure out how big the file is
		LARGE_INTEGER retFileSize;
		BOOL result = GetFileSizeEx(fileHandle, &retFileSize);
		if(result != TRUE)
		{
			CloseHandle(fileHandle);
			return ret;
		}

		ret.dataSize = retFileSize.QuadPart;

		// Allocate some memory to store the file in
		void* mem = alloc.Allocate(ret.dataSize);
		ASSERT(mem != 0);

		// Determine the size of a page so we can determine the size of the allocation
		SYSTEM_INFO sysInfo;
		GetSystemInfo(&sysInfo);
		long allocationSize = sysInfo.dwPageSize - (ret.dataSize % sysInfo.dwPageSize) + ret.dataSize;

		OVERLAPPED overlapped;
		ZeroMemory(&overlapped, sizeof(overlapped));

		overlapped.Offset = 0;
		overlapped.OffsetHigh = 0;

		// Okay, we opened it successfully...  Read it out
		overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, "ReadFile event");
		ASSERT(overlapped.hEvent != NULL);

		DWORD bytesRead;
		BOOL readResult = ReadFile(fileHandle, mem, allocationSize, &bytesRead, NULL);
		if(readResult != TRUE)
		{
			CloseHandle(overlapped.hEvent);
			CloseHandle(fileHandle);
			alloc.Free(mem);
			return ret;
		}
		//BOOL readResult = ReadFile(fileHandle, mem, allocationSize, &bytesRead, &overlapped);
		/*
		if(readResult != TRUE)
		{
			DWORD lastError = GetLastError();
			if(lastError == ERROR_IO_PENDING)
			{
				WaitForSingleObject(overlapped.hEvent,INFINITE);
				BOOL getRet = GetOverlappedResult(fileHandle, &overlapped, &bytesRead, TRUE);
				if(getRet == FALSE || bytesRead == 0)
				{
					if(bytesRead == 0)
					{
						std::cout << "Read 0 bytes" << std::endl;
					}
					CloseHandle(overlapped.hEvent);
					CloseHandle(fileHandle);
					alloc.Free(mem);
					return ret;
				}
			}
			else
			{
				CloseHandle(overlapped.hEvent);
				CloseHandle(fileHandle);
				alloc.Free(mem);
				return ret;
			}
		}
		*/

		CloseHandle(overlapped.hEvent);
		CloseHandle(fileHandle);

		// We did it!  Return it.
		ret.data = mem;

		return ret;
	}
};
