#include "Win32WindowKeyboardInput.h"

static KeyboardInput::KeyType VirtualKeyToKeyType(WPARAM wParam)
{
	switch(wParam)
	{
		case VK_ESCAPE: return KeyboardInput::KT_ESC;
		case VK_F1: return KeyboardInput::KT_F1;
		case VK_F2: return KeyboardInput::KT_F2;
		case VK_F3: return KeyboardInput::KT_F3;
		case VK_F4: return KeyboardInput::KT_F4;
		case VK_F5: return KeyboardInput::KT_F5;
		case VK_F6: return KeyboardInput::KT_F6;
		case VK_F7: return KeyboardInput::KT_F7;
		case VK_F8: return KeyboardInput::KT_F8;
		case VK_F9: return KeyboardInput::KT_F9;
		case VK_F10: return KeyboardInput::KT_F10;
		case VK_F11: return KeyboardInput::KT_F11;
		case VK_F12: return KeyboardInput::KT_F12;
		case VK_PAUSE: return KeyboardInput::KT_PAUSE;
		
		case VK_OEM_3: return KeyboardInput::KT_TILDE;
		case '1': return KeyboardInput::KT_1;
		case '2': return KeyboardInput::KT_2;
		case '3': return KeyboardInput::KT_3;
		case '4': return KeyboardInput::KT_4;
		case '5': return KeyboardInput::KT_5;
		case '6': return KeyboardInput::KT_6;
		case '7': return KeyboardInput::KT_7;
		case '8': return KeyboardInput::KT_8;
		case '9': return KeyboardInput::KT_9;
		case '0': return KeyboardInput::KT_0;
		case VK_OEM_MINUS: return KeyboardInput::KT_MINUS;
		case VK_OEM_PLUS: return KeyboardInput::KT_EQUAL;
		case VK_BACK: return KeyboardInput::KT_BACKSPACE;
		
		case VK_TAB: return KeyboardInput::KT_TAB;
		case 'Q': return KeyboardInput::KT_Q;
		case 'W': return KeyboardInput::KT_W;
		case 'E': return KeyboardInput::KT_E;
		case 'R': return KeyboardInput::KT_R;
		case 'T': return KeyboardInput::KT_T;
		case 'Y': return KeyboardInput::KT_Y;
		case 'U': return KeyboardInput::KT_U;
		case 'I': return KeyboardInput::KT_I;
		case 'O': return KeyboardInput::KT_O;
		case 'P': return KeyboardInput::KT_P;
		case VK_OEM_4: return KeyboardInput::KT_SQUARE_BRACKET_L;
		case VK_OEM_6: return KeyboardInput::KT_SQUARE_BRACKET_R;
		case VK_OEM_5: return KeyboardInput::KT_BACKSLASH;
		
		case VK_CAPITAL: return KeyboardInput::KT_CAPSLOCK;
		case 'A': return KeyboardInput::KT_A;
		case 'S': return KeyboardInput::KT_S;
		case 'D': return KeyboardInput::KT_D;
		case 'F': return KeyboardInput::KT_F;
		case 'G': return KeyboardInput::KT_G;
		case 'H': return KeyboardInput::KT_H;
		case 'J': return KeyboardInput::KT_J;
		case 'K': return KeyboardInput::KT_K;
		case 'L': return KeyboardInput::KT_L;
		case VK_OEM_1: return KeyboardInput::KT_SEMICOLON;
		case VK_OEM_7: return KeyboardInput::KT_APOSTROPHE;
		case VK_RETURN: return KeyboardInput::KT_ENTER;

		case VK_SHIFT: return KeyboardInput::KT_SHIFT;
		case 'Z': return KeyboardInput::KT_Z;
		case 'X': return KeyboardInput::KT_X;
		case 'C': return KeyboardInput::KT_C;
		case 'V': return KeyboardInput::KT_V;
		case 'B': return KeyboardInput::KT_B;
		case 'N': return KeyboardInput::KT_N;
		case 'M': return KeyboardInput::KT_M;
		case VK_OEM_COMMA: return KeyboardInput::KT_COMMA;
		case VK_OEM_PERIOD: return KeyboardInput::KT_PERIOD;
		case VK_OEM_2: return KeyboardInput::KT_SLASH;

		case VK_CONTROL: return KeyboardInput::KT_CTRL;
		case VK_SPACE: return KeyboardInput::KT_SPACE;
		
		case VK_LEFT: return KeyboardInput::KT_LEFT;
		case VK_RIGHT: return KeyboardInput::KT_RIGHT;
		case VK_UP: return KeyboardInput::KT_UP;
		case VK_DOWN: return KeyboardInput::KT_DOWN;

		case VK_INSERT: return KeyboardInput::KT_INSERT;
		case VK_DELETE: return KeyboardInput::KT_DELETE;
		case VK_HOME: return KeyboardInput::KT_HOME;
		case VK_END: return KeyboardInput::KT_END;
		case VK_PRIOR: return KeyboardInput::KT_PG_UP;
		case VK_NEXT: return KeyboardInput::KT_PG_DOWN;

		default: return KeyboardInput::KT_INVALID;
	};
}

void Win32WindowKeyboardInput::PostKeyDownEvent(WPARAM wParam, LPARAM lParam)
{
	if(lParam & (1 << 30)) return; // Ignore repeat keys

	KeyboardInput::KeyEvent nextEvent;

	nextEvent.key = VirtualKeyToKeyType(wParam);
	if(nextEvent.key == KeyboardInput::KT_INVALID) return;

	nextEvent.pressed = true;

	PostKeyPress(nextEvent);
}

void Win32WindowKeyboardInput::PostKeyUpEvent(WPARAM wParam, LPARAM lParam)
{
	KeyboardInput::KeyEvent nextEvent;

	nextEvent.key = VirtualKeyToKeyType(wParam);
	if(nextEvent.key == KeyboardInput::KT_INVALID) return;

	nextEvent.pressed = false;

	PostKeyPress(nextEvent);
}

