#include "Win32WindowMouseInput.h"

#include <windows.h>

#include <iostream>
#include <boost/bind.hpp>

void Win32WindowMouseInput::Run()
{
	{
		Mutex::ScopedLock lock(m_monitorLock);

		HRESULT hr;

		hr = DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&m_di, NULL);
		ASSERT(!FAILED(hr));

		hr = m_di->CreateDevice(GUID_SysMouse, &m_diMouse, NULL);
		ASSERT(!FAILED(hr));

		hr = m_diMouse->SetDataFormat(&c_dfDIMouse2);
		ASSERT(!FAILED(hr));

		hr = m_diMouse->SetCooperativeLevel(m_hwnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
		ASSERT(!FAILED(hr));

		m_mouseEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
		ASSERT(m_mouseEvent != NULL);

		hr = m_diMouse->SetEventNotification(m_mouseEvent);
		ASSERT(!FAILED(hr));

		hr = m_diMouse->Acquire();

		m_setupDoneCond.SignalAll();
	}

	while(true)
	{
		WaitForMultipleObjects(1, &m_mouseEvent, FALSE, INFINITE);
		ResetEvent(m_mouseEvent);

		if(m_quitting)
		{
			break;
		}

		// Process mouse state update

		HRESULT hr;

		DIMOUSESTATE2 dims2;
		ZeroMemory(&dims2, sizeof(dims2));
	    hr = m_diMouse->GetDeviceState( sizeof(DIMOUSESTATE2), &dims2 );
		if(FAILED(hr))
		{
			do
			{
				WaitForMultipleObjects(1, &m_mouseEvent, FALSE, 500); // We're probably inactive, so sleep 500 ms

				if(m_quitting) break;
				hr = m_diMouse->Acquire();
			} while(hr != DI_OK && hr != S_FALSE);

			continue;
		}

		MoveEvent moveEvent;
		moveEvent.dx = dims2.lX;
		moveEvent.dy = dims2.lY;
		PostMoveEvent(moveEvent);

		for(int i=0; i<NUM_BUTTONS; ++i)
		{
			if(m_buttonStates[i] != (dims2.rgbButtons[i] & 0x80 ? true : false))
			{
				m_buttonStates[i] = !m_buttonStates[i];

				ButtonEvent buttonEvent;
				buttonEvent.button = i;
				buttonEvent.pressed = m_buttonStates[i];

				PostButtonEvent(buttonEvent);
			}
		}
	}
}


Win32WindowMouseInput::Win32WindowMouseInput(HWND hwnd)
: m_di(NULL)
, m_diMouse(NULL)
, m_hwnd(hwnd)
, m_quitting(false)
{
	Mutex::ScopedLock lock(m_monitorLock);

	for(int i=0; i<NUM_BUTTONS; ++i)
	{
		m_buttonStates[i] = false;
	}

	m_thread = new Thread(boost::bind(&Win32WindowMouseInput::Run, this));

	m_setupDoneCond.Wait(m_monitorLock);
}

Win32WindowMouseInput::~Win32WindowMouseInput()
{
	m_quitting = true;
	SetEvent(m_mouseEvent);

	delete m_thread;
}

void Win32WindowMouseInput::AttemptReaquire()
{
	SetEvent(m_mouseEvent);
}
