#ifndef MYALLOCATOR_H_
#define MYALLOCATOR_H_

#include <ThreadPrimitives.h>

#include <malloc.h>

class Allocator
{
public:
	virtual void* Allocate(long bytes) = 0;
	virtual void Free(void* mem) = 0;
};

// Used to suggest that the allocator will be used by multiple threads
class ThreadSafeAllocator : public Allocator
{
};

// Thread safe decorator
class LockingAllocatorDecorator : public ThreadSafeAllocator
{
	Allocator& m_alloc;
	Mutex m_mutex;

public:
	LockingAllocatorDecorator(Allocator& alloc)
		: m_alloc(alloc)
	{};

	virtual void* Allocate(long bytes)
	{
		Mutex::ScopedLock lock(m_mutex);
		return m_alloc.Allocate(bytes);
	}
	virtual void Free(void* mem)
	{
		Mutex::ScopedLock lock(m_mutex);
		m_alloc.Free(mem);
	}
};

class StandardAllocator : public Allocator
{
public:
	virtual void* Allocate(long bytes)
	{
		return malloc(bytes);
	}
	virtual void Free(void* mem)
	{
		free(mem);
	}
};
extern StandardAllocator g_standardAllocator;

void* operator new(unsigned int num_bytes, Allocator& alloc);
void* operator new[](unsigned int num_bytes, Allocator& alloc);
void operator delete(void* ptr, Allocator& alloc);
void operator delete[](void* ptr, Allocator& alloc);


#endif
