#ifndef TOP_GENERIC_WINDOW_H_
#define TOP_GENERIC_WINDOW_H_

class GraphicsContext;
class KeyboardInput;
class MouseInput;

#include <Utils.h>

#include <Events.h>

class GenericWindow
{
public:
	Event<void ()> m_quitEvent;

protected:
	void FireQuitEvent() {m_quitEvent.Fire();} // The quit event should be fired whenever the window decides on its own that it wants to quit (ie: someone clicks on the top right X)

public:
	GenericWindow()
	{};
	virtual ~GenericWindow() {};

	// Input
	virtual GraphicsContext& GetGraphicsContext() = 0;

	// Output
	virtual KeyboardInput& GetKeyboardInput() = 0;
	virtual MouseInput& GetMouseInput() = 0;
	struct MousePosition
	{
		int x;
		int y;
		MousePosition(int pX, int pY)
			: x(pX)
			, y(pY)
		{};
	};
	virtual MousePosition GetLastMousePosition() = 0; // Returns the last position the mouse was in on this window

	virtual float GetWidth() = 0;
	virtual float GetHeight() = 0;

	Event<void ()>::Port GetQuitEventPort()
	{
		return m_quitEvent.GetPort();
	}
};

#endif
