#ifndef RESOURCEIDENTIFIER_H_
#define RESOURCEIDENTIFIER_H_

#include <boost/filesystem.hpp>
#include <string>

class ResourceIdentifier
{
	boost::filesystem::path m_filePath;

	std::string StripTrailingWhitespaces(const std::string& str)
	{
		std::string ret = str;
		while(!ret.empty() && 
			(*ret.rbegin() == '\r' || *ret.rbegin() == '\n' || *ret.rbegin() == '\t' || *ret.rbegin() == ' '))
		{
			ret.erase(ret.size()-1);
		}
		return ret;
	}

public:
	ResourceIdentifier(const boost::filesystem::path& path)
		: m_filePath(path)
	{};
	ResourceIdentifier(const char* filename)
		: m_filePath(StripTrailingWhitespaces(filename))
	{};
	ResourceIdentifier(const std::string& filename)
		: m_filePath(StripTrailingWhitespaces(filename))
	{};
	ResourceIdentifier(const ResourceIdentifier& other)
	{
		operator=(other);
	}
	const ResourceIdentifier& operator=(const ResourceIdentifier& other)
	{
		if(&other != this)
		{
			m_filePath = other.m_filePath;
		}
		return *this;
	}

	bool operator<(const ResourceIdentifier& other) const
	{
		return m_filePath < other.m_filePath;
	}

	const boost::filesystem::path& GetFilepath() const {return m_filePath;}

	// Is this ResourceIdentifier relative to another resource?
	bool IsRelative() const
	{
		return m_filePath.root_path().empty();
	}

	ResourceIdentifier Append(const ResourceIdentifier& next) const
	{
		return ResourceIdentifier(m_filePath.branch_path() / next.m_filePath);
	}
};

#endif
