#ifndef TRIANGLEMESH_H_
#define TRIANGLEMESH_H_

#include <Algebra.h>
#include <vector>
/*
	The TriangleMesh class represents a mesh of triangles, along with
	the vertices of the triangles and any properties of those vertices
	(such as normals or UV coords)
*/

class TriangleMesh
{
public:
	typedef Vector<3, VT_INSTANCE, false> Vertex;
	typedef std::vector<Vertex> VertexArray;

	class Triangle
	{
		int m_vertices[3];

		typedef Vector<3, VT_INSTANCE, false> Normal;
		Normal m_normals[3];

		bool m_hasNormals;

	public:
		Triangle(int vert1, int vert2, int vert3)
			: m_hasNormals(false)
		{
			m_vertices[0] = vert1;
			m_vertices[1] = vert2;
			m_vertices[2] = vert3;
		}
	
		int GetVertex(int index) const {ASSERT(index >=0); ASSERT(index <3); return m_vertices[index];}
		const Normal& GetNormal(int index) const {ASSERT(m_hasNormals); ASSERT(index >=0); ASSERT(index <3); return m_normals[index];}
		bool HasNormals() const {return m_hasNormals;}

		void SetNormals(const Normal& norm1, const Normal& norm2, const Normal& norm3)
		{
			m_normals[0] = norm1;
			m_normals[1] = norm2;
			m_normals[2] = norm3;
			m_hasNormals = true;
		}
	};
	typedef std::vector<Triangle> TriangleArray;

private:
	VertexArray m_vertices;
	TriangleArray m_triangles;

public:
	TriangleMesh();

	void FillInNormals(); // Sometimes normals don't get set.  This function generates them for each face (flat shading)

	int AddVertex(const Vertex& pos); // Adds a vertex with the given position, returns the vertex id
	void AddTriangle(const Triangle& tri);

	const TriangleArray& GetTriangles() const {return m_triangles;}
	const VertexArray& GetVertices() const {return m_vertices;}
};

#endif
