#include <R3Utils.h>

Matrix<4,4> MakeIdentityMatrix()
{
	Matrix<4,4> ret;
	ret.GetVal(0) = 1;
	ret.GetVal(1) = 0;
	ret.GetVal(2) = 0;
	ret.GetVal(3) = 0;
	ret.GetVal(4) = 0;
	ret.GetVal(5) = 1;
	ret.GetVal(6) = 0;
	ret.GetVal(7) = 0;
	ret.GetVal(8) = 0;
	ret.GetVal(9) = 0;
	ret.GetVal(10) = 1;
	ret.GetVal(11) = 0;
	ret.GetVal(12) = 0;
	ret.GetVal(13) = 0;
	ret.GetVal(14) = 0;
	ret.GetVal(15) = 1;
	return ret;
}

Matrix<4,4> MakeZeroMatrix()
{
	Matrix<4,4> ret;
	ret.GetVal(0) = 0;
	ret.GetVal(1) = 0;
	ret.GetVal(2) = 0;
	ret.GetVal(3) = 0;
	ret.GetVal(4) = 0;
	ret.GetVal(5) = 0;
	ret.GetVal(6) = 0;
	ret.GetVal(7) = 0;
	ret.GetVal(8) = 0;
	ret.GetVal(9) = 0;
	ret.GetVal(10) = 0;
	ret.GetVal(11) = 0;
	ret.GetVal(12) = 0;
	ret.GetVal(13) = 0;
	ret.GetVal(14) = 0;
	ret.GetVal(15) = 0;
	return ret;
}

Matrix<4,4> MakeUniScaleMatrix(float scale)
{	
	Matrix<4,4> ret;
	ret.GetVal(0) = scale;
	ret.GetVal(1) = 0;
	ret.GetVal(2) = 0;
	ret.GetVal(3) = 0;
	ret.GetVal(4) = 0;
	ret.GetVal(5) = scale;
	ret.GetVal(6) = 0;
	ret.GetVal(7) = 0;
	ret.GetVal(8) = 0;
	ret.GetVal(9) = 0;
	ret.GetVal(10) = scale;
	ret.GetVal(11) = 0;
	ret.GetVal(12) = 0;
	ret.GetVal(13) = 0;
	ret.GetVal(14) = 0;
	ret.GetVal(15) = 1;
	return ret;
}

Matrix<4,4> MakeTranslationMatrix(const Vector<3>& trans)
{
	Matrix<4,4> ret;
	ret.GetVal(0) = 1;
	ret.GetVal(1) = 0;
	ret.GetVal(2) = 0;
	ret.GetVal(3) = 0;
	ret.GetVal(4) = 0;
	ret.GetVal(5) = 1;
	ret.GetVal(6) = 0;
	ret.GetVal(7) = 0;
	ret.GetVal(8) = 0;
	ret.GetVal(9) = 0;
	ret.GetVal(10) = 1;
	ret.GetVal(11) = 0;
	ret.GetVal(12) = trans[0];
	ret.GetVal(13) = trans[1];
	ret.GetVal(14) = trans[2];
	ret.GetVal(15) = 1;
	return ret;
}

Matrix<4,4> MakeRotationMatrix(float angle, char axis)
{
	double st = sin(angle);
	double ct = cos(angle);

	Matrix<4,4> ret(MakeIdentityMatrix());

	if(axis == 'x' || axis == 'X')
	{
		ret.GetVal(1,1) = ct;
		ret.GetVal(2,1) = st;
		ret.GetVal(1,2) = -st;
		ret.GetVal(2,2) = ct;
	}
	else if(axis == 'y' || axis == 'Y')
	{
		ret.GetVal(0,0) = ct;
		ret.GetVal(2,0) = -st;
		ret.GetVal(0,2) = st;
		ret.GetVal(2,2) = ct;
	}
	else if(axis == 'z' || axis == 'Z')
	{
		ret.GetVal(0,0) = ct;
		ret.GetVal(1,0) = st;
		ret.GetVal(0,1) = -st;
		ret.GetVal(1,1) = ct;
	}

	return ret;
}

