#ifndef RENDERANALYZER
#define RENDERANALYZER

#include "WorldInstance.h"

#include <GraphicsFrame.h>

class BoundingBoxPropertiesAnalyzer;

void MakeTransformedCubePoints(const Matrix<4,4>& trans, Vector<4>* cubePoints);

class RenderAnalyzer
{
	private:

	GraphicsFrame& m_frame;
	Matrix<4,4> m_projection;
	float m_near;


	bool PointsWithinBox(const Vector<4>* points, int numPoints) const;
	float GetMaxScreenSpaceArea(const Vector<4> cubePoints[8]) const;
	float GetAlphaForArea(float area, float areaThreshold);

	public:
	RenderAnalyzer(GraphicsFrame& frame, const Matrix<4,4>& proj, float near)
	: m_frame(frame)
	, m_projection(proj)
	, m_near(near)
	{};

	struct CustomParams
	{
		float alpha;

		CustomParams()
			: alpha(1.0f)
		{};
	};

	bool Analyze(SymbolInstance* curInstance, const Matrix<4,4>& modelView, const BoundingBoxProperties& boxProps, const CustomParams& inParams, CustomParams& outParams, int layer);
};

#endif
