#ifndef SYMBOLVIEWPANEL_H_
#define SYMBOLVIEWPANEL_H_

#include "ToolWorldDefinition.h"

#include <string>

#include <Events.h>

#include <wx/wx.h>
#include <WorldDefinition.h>
#include <PotentialObject.h>

class SymbolViewPanel : public wxPanel
{
	enum
	{
		APPROX_WITH_BUTTON = 200,
		CLEAR_APPROX_WITH_BUTTON,
		APPROX_THRESHOLD,
		ADD_PRODUCTION_BUTTON,
		REM_PRODUCTION_BUTTON,

		MY_PRODS_LIST,
		PROD_APART_OF_LIST,
	};

	wxButton* m_approxThresh;
	wxButton* m_approxWith;
	wxButton* m_clearApproxWith;

	wxListBox* m_prodsList;
	wxButton* m_addProdButton;
	wxButton* m_remProdButton;

	wxListBox* m_prodsApartOfList;

	void OnSelectApproxWith(wxCommandEvent&);
	void OnSelectApproxThresh(wxCommandEvent&);
	void OnClearApproxWith(wxCommandEvent&);

	void OnAddProd(wxCommandEvent&);
	void OnRemoveProd(wxCommandEvent&);

	void OnMyProdsDClick(wxCommandEvent&);
	void OnProdsApartOfDClick(wxCommandEvent&);
	void OnProductionListDoubleclicked(wxListBox* list);

	ToolWorldDefinition& m_world;
	std::string m_symName;

	const ToolSymbol* m_sym;

	void AddProductionToListbox(const Production* prod, wxListBox* listBox);
	void RemoveProductionFromListbox(const Production* prod, wxListBox* listBox);

	PotentialObject<Event<ToolWorldDefinition::ProductionRemovedSig>::Connection> m_prodRemovedConn;
	void HandleProductionRemoved(const ToolProduction* prod);

	PotentialObject<Event<ToolWorldDefinition::ProductionAddedSig>::Connection> m_prodAddedConn;
	void HandleProductionAdded(const ToolProduction* prod);

	PotentialObject<Event<ToolWorldDefinition::ProductionRSAddedSig>::Connection> m_prodRSAddedConn;
	PotentialObject<Event<ToolWorldDefinition::ProductionRSRemovedSig>::Connection> m_prodRSRemovedConn;
	void HandleProductionRSAdded(const ToolProduction* prod);
	void HandleProductionRSRemoved(const ToolProduction* prod);

	void RefreshProdsApartOfList();

	PotentialObject<Event<ToolWorldDefinition::SymbolChangedSig>::Connection> m_symbolChangedConn;
	void HandleSymbolChanged(const ToolSymbol* sym);

	void RefreshProductionInListbox(const Production* prod, wxListBox* list);

	wxFrame* m_parentFrame;

	void RefreshSymbolProperties();

public:
	SymbolViewPanel(ToolWorldDefinition& world, const std::string& symName, wxWindow* parent, wxFrame* parentFrame);
	virtual ~SymbolViewPanel();

	const std::string& GetSymbolName() const {return m_symName;}
	
	DECLARE_EVENT_TABLE()
};

#endif
