#ifndef WXWIDGETMOUSEINPUT_H_
#define WXWIDGETMOUSEINPUT_H_

#include <wx/wx.h>
#include <MouseInput.h>

#include <PotentialObject.h>

#include <set>

/** Converts wxWidget mouse events in to the generic format presented by the MouseInput class.
 *	Anything that interfaces with MouseInput can now accept wxWidget events.
 */
class wxWidgetMouseInput : public MouseInput
{
	class EventHandler : public wxEvtHandler
	{
		wxWidgetMouseInput& m_parent;

		void OnMouseMoved(wxMouseEvent& event);
		void OnLeftDown(wxMouseEvent& event);
		void OnLeftUp(wxMouseEvent& event);
		void OnRightDown(wxMouseEvent& event);
		void OnRightUp(wxMouseEvent& event);

	public:
		EventHandler(wxWidgetMouseInput& parent)
			: m_parent(parent)
		{};

		DECLARE_EVENT_TABLE()
	};
	PotentialObject<EventHandler> m_handler;

	void OnMouseMoved(wxMouseEvent& event);
	void OnLeftDown(wxMouseEvent& event);
	void OnLeftUp(wxMouseEvent& event);
	void OnRightDown(wxMouseEvent& event);
	void OnRightUp(wxMouseEvent& event);

	int m_lastX;
	int m_lastY;

public:
	/**
	 *	@param handler The wxWidgetMouseInput object will link in with the passed in wxWidget
	 *					event handler's event handling list.  We will then listen for mouse events.
	 */
	wxWidgetMouseInput(wxEvtHandler& handler);
};

#endif
